{***************************************************************************}
{ TMS GuiMotions Demo                                                       }
{ for Delphi & C++Builder                                                   }
{                                                                           }
{ written by TMS Software                                                   }
{            copyright  2008                                               }
{            Email : info@tmssoftware.com                                   }
{            Web : http://www.tmssoftware.com                               }
{                                                                           }
{ The source code is given as is. The author is not responsible             }
{ for any possible damage done due to the use of this code.                 }
{ The component can be freely used in any application. The complete         }
{ source code remains property of the author and may not be distributed,    }
{ published, given or sold in any form as such. No parts of the source      }
{ code can be included in any other component or application without        }
{ written authorization of the author.                                      }
{***************************************************************************}

{
  - Library PngImage must be present and Delphi library path must be updated (http://www.torry.net/vcl/graphics/otherformats/pngimage.zip)
  - d3dx9_33.dll must be present in the \Windows\System32 folder
}

unit GuimotionsEx;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, GUIMotions, PngImage, Guimotions3D, FileCtrl, ShellAPI, StdCtrls, ExtCtrls, ShlObj,
  ComCtrls;

type
  TForm1 = class(TForm)
    GUIMotions1: TGUIMotions;
    Panel1: TPanel;
    LoadImages: TCheckBox;
    Button4: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    TrackBar1: TTrackBar;
    Label4: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure GUIMotions1ImageDblClick(Sender: TObject; Image: Integer);
    procedure Button4Click(Sender: TObject);
    procedure GUIMotions1ImageLeftClick(Sender: TObject; Image: Integer;
      var AllowZoom: Boolean; device: IDirect3DDevice9);
    procedure GUIMotions1ImageRightClick(Sender: TObject; Image: Integer;
      var AllowAlternate: Boolean; device: IDirect3DDevice9);
    procedure TrackBar1Change(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure PopulateDirectory(AFolder: string);
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}
{$R GuimotionsEx.res}

function GetParentDirectory(path : string) : string;
begin
  result := ExpandFileName(path + '\..')
end;

function AddFilter(path,filter: string): string;
begin
  if length(path) > 0 then
  begin
    if path[length(path)] = '\' then
      result := path + filter
    else
      result := path + '\' + filter;
  end
  else
   result := filter;

end;


procedure TForm1.GUIMotions1ImageDblClick(Sender: TObject; Image: Integer);
var
  newdir: string;
begin
  GUIMotions1.BeginUpdate;
  if Image < guimotions1.Images.Count then
  begin
    if guimotions1.Images[Image].Tag and faDirectory = faDirectory then
    begin
      newdir := guimotions1.Images[Image].Data;
      Guimotions1.Images.Clear;
      PopulateDirectory(AddFilter(newdir,'*.*'));
      Guimotions1.Reset;
      Guimotions1.SelectImage(0);
    end
    else
      ShellExecute(0,'open',pchar(guimotions1.Images[Image].Data),nil,nil,SW_NORMAL);
  end;
  GUIMotions1.EndUpdate;
end;

procedure TForm1.GUIMotions1ImageLeftClick(Sender: TObject; Image: Integer;
  var AllowZoom: Boolean; device: IDirect3DDevice9);
begin
  AllowZoom := false;
end;

procedure TForm1.GUIMotions1ImageRightClick(Sender: TObject; Image: Integer;
  var AllowAlternate: Boolean; device: IDirect3DDevice9);
begin
  AllowAlternate := false;
end;

procedure TForm1.PopulateDirectory(AFolder: string);
var
  SR: TSearchRec;

  procedure LoadFromRes(resname: string; picture: TPicture);
  var
    APNG: TPNGObject;
    rs: TResourceStream;
  begin
    rs := TResourceStream.Create(HInstance, resname, RT_RCDATA);
    APNG := TPNGObject.Create;
    APNG.LoadFromStream(rs);
    rs.Free;
    picture.Assign(APNG);
    APNG.Free;
  end;

  procedure AddToList(s: string; Attr: integer);
  var
    fext: string;

  begin
    if not Attr and faHidden = faHidden then
    begin

      with Guimotions1.Images.Add do
      begin
        if ExtractFileName(s) = '.' then
        begin
          Data := ExtractFilePath(s);
          Caption := Data;
        end
        else
          if ExtractFileName(s) = '..' then
          begin
            Data := GetParentDirectory(ExtractFilepath(AFolder));
            Caption := Data;
          end
          else
          begin
            Data := s;
            Caption := ExtractFileName(s);
          end;

        Tag := Attr;

        if Attr and faDirectory = faDirectory then
        begin
          LoadFromRes('file',picture);
        end
        else
        begin
          //if (Attr and faArchive = faArchive then
          begin
            fext := Uppercase(ExtractFileExt(s));

            if LoadImages.Checked and ((UpperCase(fext) = '.PNG') or (UpperCase(fext) = '.JPG') or (UpperCase(fext) = '.JPEG') or (UpperCase(fext) = '.BMP')) then
            begin
              ThreadedPictureLocation := s;
              ThreadStatus := stInitialized;
            end
            else
            begin
              LoadFromRes('document',picture);

              if UpperCase(fext) = '.TXT' then
                LoadFromRes('notepad',picture);

              if UpperCase(fext) = '.BMP' then
                LoadFromRes('image',picture);

              if (UpperCase(fext) = '.JPG') or (UpperCase(fext) = '.JPEG') then
                LoadFromRes('image',picture);

              if UpperCase(fext) = '.GIF' then
                LoadFromRes('image',picture);

              if UpperCase(fext) = '.PNG' then
                LoadFromRes('image',picture);

              if UpperCase(fext) = '.MP3' then
                LoadFromRes('music',picture);

              if UpperCase(fext) = '.WAV' then
                LoadFromRes('music',picture);

              if UpperCase(fext) = '.MSG' then
                LoadFromRes('message',picture);

              if UpperCase(fext) = '.HTML' then
                LoadFromRes('html',picture);

              if UpperCase(fext) = '.HTM' then
                LoadFromRes('html',picture);

              if UpperCase(fext) = '.DOC' then
                LoadFromRes('doc',picture);

              if UpperCase(fext) = '.PDF' then
                LoadFromRes('pdf',picture);

              if UpperCase(fext) = '.EXE' then
                LoadFromRes('exe',picture);

              if UpperCase(fext) = '.ZIP' then
                LoadFromRes('zip',picture);

              if UpperCase(fext) = '.XLS' then
                LoadFromRes('excel',picture);

              if UpperCase(fext) = '.XLSX' then
                LoadFromRes('excel',picture);

              if UpperCase(fext) = '.PPT' then
                LoadFromRes('powerpoint',picture);

              if UpperCase(fext) = '.PAS' then
                LoadFromRes('delphi',picture);

              if UpperCase(fext) = '.DPR' then
                LoadFromRes('delphi',picture);
            end;
          end
          //else
          //  LoadFromRes('document',picture);
        end;
      end;
    end;
  end;

begin
  if FindFirst(AFolder,faAnyFile,SR) = 0 then
  begin
    AddToList(ExtractFilePath(AFolder) + SR.Name, SR.Attr);
    while FindNext(SR) = 0 do
    begin
      AddToList(ExtractFilePath(AFolder) + SR.Name, SR.Attr);
    end;
  end;
  FindClose(SR);
end;


procedure TForm1.TrackBar1Change(Sender: TObject);
begin
  GUIMotions1.SelectedScaleFactor := TrackBar1.Position;
end;

function PathToMyDocuments: string;
var
  Path : pchar;
  idList : PItemIDList;
begin
  GetMem(Path, MAX_PATH);
  SHGetSpecialFolderLocation(0, CSIDL_PERSONAL, idList);
  SHGetPathFromIDList(idList, Path);
  Result := string(Path);
  FreeMem(Path);
end;

procedure TForm1.Button4Click(Sender: TObject);
var
  dir: String;
begin
  if SelectDirectory('Select a directory', '', dir) then
  begin
    GUIMotions1.Images.Clear;
    PopulateDirectory(dir + '\*.*');
    Guimotions1.Reset;
    GUIMotions1.SetFocus;   
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  LoadImages.Checked := true;
//  populatedirectory(PathToMyDocuments + '\*.*');
  GUIMotions1.KeyboardLookup := true;
  TrackBar1.Position := Round(GUIMotions1.SelectedScaleFactor);
end;

end.
